


setwd("C:/Users/ksb11175/Strathcloud/Personal Folders/Work/Teaching/EES-UETP/")

load("WindData.Rda")

WindData$Spd100 <- sqrt(WindData$U100^2 + WindData$V100^2)



## Simple Linear Model ####

lm_simple <- lm(Power~Spd100,data = WindData)

ModelPlotData <- data.frame(Spd100=seq(0,30,by=0.1))
ModelPlotData$lm_simple <- predict(lm_simple,newdata = ModelPlotData)

plot(Power~Spd100,WindData,pch=".")
lines(lm_simple~Spd100,ModelPlotData,lwd=2,col="red")

WindData$lm_simple <- predict(lm_simple,newdata = WindData)

plot(Power~TARGETdtm,WindData[1:240,],type="l",xlab="Date/Time")
lines(lm_simple~TARGETdtm,WindData[1:240,],col="red",lty=2)
legend("top",c("Actual","Prediction"),lty=1:2,col = c(1,"red"),bty = "n")

## Spline vs Penalised Spline
require(mgcv)
require(splines)
gam_1 <- lm(Power~bs(Spd100,,knots = seq(0,20,by=0.5)),data=WindData)
gam_2 <- gam(Power~s(Spd100,bs="cs"),data=WindData)

ModelPlotData$gam_1 <- predict(gam_1,newdata = ModelPlotData)
ModelPlotData$gam_2 <- predict(gam_2,newdata = ModelPlotData)

plot(Power~Spd100,WindData,pch=".")
lines(gam_1~Spd100,ModelPlotData,lwd=2,col="red")
lines(gam_2~Spd100,ModelPlotData,lwd=2,col="blue")


## Simple GAMLSS ####
require(gamlss)
WindDataGAMLSS <- na.omit(WindData[,c(3,8)])

gamlss_1 <- gamlss(Power~Spd100,data=WindDataGAMLSS,
                   sigma.formula = ~1,
                   family = NO)


ModelPlotData$gamlss_1 <- predict(gamlss_1,what = "mu",newdata = data.frame(Spd100=ModelPlotData[["Spd100"]]),data = WindDataGAMLSS)
ModelPlotData$gamlss_1sigma <- exp(predict(gamlss_1,what = "sigma",newdata = data.frame(Spd100=ModelPlotData[["Spd100"]]),data = WindDataGAMLSS))

plot(Power~Spd100,WindData,pch=".")
qs=1:9/10
for(p in qs[1:floor(length(qs)/2)]){
  polygon(c(ModelPlotData$Spd100,rev(ModelPlotData$Spd100)),
          c(qNO(p = p,
                mu = ModelPlotData$gamlss_1,
                sigma = ModelPlotData$gamlss_1sigma),
            rev(qNO(p = 1-p,
                    mu = ModelPlotData$gamlss_1,
                    sigma = ModelPlotData$gamlss_1sigma))),border = F,col = adjustcolor("blue",alpha.f = p))
}

ModelPlotData_ts <- WindData[1:240,c(2,3,8)]
ModelPlotData_ts$gamlss_1 <- predict(gamlss_1,what = "mu",newdata = data.frame(Spd100=ModelPlotData_ts[["Spd100"]]),data = WindDataGAMLSS)
ModelPlotData_ts$gamlss_1sigma <- exp(predict(gamlss_1,what = "sigma",newdata = data.frame(Spd100=ModelPlotData_ts[["Spd100"]]),data = WindDataGAMLSS))

plot(Power~TARGETdtm,ModelPlotData_ts[1:240,],type="l",xlab="Date/Time")
qs=1:9/10
for(p in qs[1:floor(length(qs)/2)]){
  polygon(c(ModelPlotData_ts$TARGETdtm,rev(ModelPlotData_ts$TARGETdtm)),
          c(qNO(p = p,
                mu = ModelPlotData_ts$gamlss_1,
                sigma = ModelPlotData_ts$gamlss_1sigma),
            rev(qNO(p = 1-p,
                    mu = ModelPlotData_ts$gamlss_1,
                    sigma = ModelPlotData_ts$gamlss_1sigma))),border = F,col = adjustcolor("blue",alpha.f = p))
}
lines(Power~TARGETdtm,ModelPlotData_ts[1:240,])
legend("top",c("Actual","Prediction"),lty=c(1,NA),pch=c(NA,15),col = c(1,"blue"),bty = "n")


## Cubic Spline GAMLSS ####

gamlss_2 <- gamlss(Power~cs(Spd100),data=WindDataGAMLSS,
                   sigma.formula = ~Spd100, # or ~1 for simple variance
                   family = NO)

ModelPlotData$gamlss_2 <- predict(gamlss_2,what = "mu",newdata = data.frame(Spd100=ModelPlotData[["Spd100"]]),data = WindDataGAMLSS)
ModelPlotData$gamlss_2sigma <- exp(predict(gamlss_2,what = "sigma",newdata = data.frame(Spd100=ModelPlotData[["Spd100"]]),data = WindDataGAMLSS))

plot(Power~Spd100,WindData,pch=".")
qs=1:9/10
for(p in qs[1:floor(length(qs)/2)]){
  polygon(c(ModelPlotData$Spd100,rev(ModelPlotData$Spd100)),
          c(qNO(p = p,
                mu = ModelPlotData$gamlss_2,
                sigma = ModelPlotData$gamlss_2sigma),
            rev(qNO(p = 1-p,
                    mu = ModelPlotData$gamlss_2,
                    sigma = ModelPlotData$gamlss_2sigma))),border = F,col = adjustcolor("blue",alpha.f = p))
}


## Cubic Spline GAMLSS, Inflated Beta ####
require(gamlss.dist)

gamlss_3 <- gamlss(Power~cs(Spd100),data=WindDataGAMLSS,
                   sigma.formula = ~cs(Spd100), # or ~1 for simple variance
                   nu.formula = ~Spd100,
                   tau.formula = ~Spd100,
                   family = BEINF)

ModelPlotData$gamlss_3 <- 1/(1+exp(-predict(gamlss_3,what = "mu",newdata = data.frame(Spd100=ModelPlotData[["Spd100"]]),data = WindDataGAMLSS)))
ModelPlotData$gamlss_3sigma <- 1/(1+exp(-predict(gamlss_3,what = "sigma",newdata = data.frame(Spd100=ModelPlotData[["Spd100"]]),data = WindDataGAMLSS)))
ModelPlotData$gamlss_3nu <- exp(predict(gamlss_3,what = "nu",newdata = data.frame(Spd100=ModelPlotData[["Spd100"]]),data = WindDataGAMLSS))
ModelPlotData$gamlss_3tau <- exp(predict(gamlss_3,what = "tau",newdata = data.frame(Spd100=ModelPlotData[["Spd100"]]),data = WindDataGAMLSS))

plot(Power~Spd100,WindData,pch=".")
qs=1:9/10
for(p in qs[1:floor(length(qs)/2)]){
  polygon(c(ModelPlotData$Spd100,rev(ModelPlotData$Spd100)),
          c(qBEINF(p = p,
                   mu = ModelPlotData$gamlss_3,
                   sigma = ModelPlotData$gamlss_3sigma,
                   nu = ModelPlotData$gamlss_3nu,
                   tau = ModelPlotData$gamlss_3tau),
            rev(qBEINF(p = 1-p,
                       mu = ModelPlotData$gamlss_3,
                       sigma = ModelPlotData$gamlss_3sigma,
                       nu = ModelPlotData$gamlss_3nu,
                       tau = ModelPlotData$gamlss_3tau))),border = F,col = adjustcolor("blue",alpha.f = p))
}

i <- 30
x <- seq(0,1,by=0.001)
y <- pBEINF(x,mu = ModelPlotData$gamlss_3[i],
            sigma = ModelPlotData$gamlss_3sigma[i],
            nu = ModelPlotData$gamlss_3nu[i],
            tau = ModelPlotData$gamlss_3tau[i])
plot(x,y,type="l",ylim=c(0,1))

ModelPlotData_ts <- WindData[1:240,c(2,3,8)]
ModelPlotData_ts$gamlss_3 <- 1/(1+exp(-predict(gamlss_3,what = "mu",newdata = data.frame(Spd100=ModelPlotData_ts[["Spd100"]]),data = WindDataGAMLSS)))
ModelPlotData_ts$gamlss_3sigma <- 1/(1+exp(-predict(gamlss_3,what = "sigma",newdata = data.frame(Spd100=ModelPlotData_ts[["Spd100"]]),data = WindDataGAMLSS)))
ModelPlotData_ts$gamlss_3nu <- exp(predict(gamlss_3,what = "nu",newdata = data.frame(Spd100=ModelPlotData_ts[["Spd100"]]),data = WindDataGAMLSS))
ModelPlotData_ts$gamlss_3tau <- exp(predict(gamlss_3,what = "tau",newdata = data.frame(Spd100=ModelPlotData_ts[["Spd100"]]),data = WindDataGAMLSS))

plot(Power~TARGETdtm,ModelPlotData_ts[1:240,],type="l",xlab="Date/Time")
qs=1:9/10
for(p in qs[1:floor(length(qs)/2)]){
  polygon(c(ModelPlotData_ts$TARGETdtm,rev(ModelPlotData_ts$TARGETdtm)),
          c(qBEINF(p = p,
                   mu = ModelPlotData_ts$gamlss_3,
                   sigma = ModelPlotData_ts$gamlss_3sigma,
                   nu = ModelPlotData_ts$gamlss_3nu,
                   tau = ModelPlotData_ts$gamlss_3tau),
            rev(qBEINF(p = 1-p,
                       mu = ModelPlotData_ts$gamlss_3,
                       sigma = ModelPlotData_ts$gamlss_3sigma,
                       nu = ModelPlotData_ts$gamlss_3nu,
                       tau = ModelPlotData_ts$gamlss_3tau))),border = F,col = adjustcolor("blue",alpha.f = p))
}
lines(Power~TARGETdtm,ModelPlotData_ts[1:240,])
legend("top",c("Actual","Prediction"),lty=c(1,NA),pch=c(NA,15),col = c(1,"blue"),bty = "n")




## Simple Decision Tree ####
require(rpart)
require(rpart.plot)

WindDataTREES <- WindData[!is.na(WindData$Power),]

Train <- 1:(nrow(WindDataTREES)/2)
tree1mse <- data.frame(CP=10^seq(0,-3,by=-0.1),
                       mse=NA,
                       nsplit=NA)
for(CPi in 1:length(tree1mse$CP)){
  
  tree1 <- rpart(Power~Spd100+U100+V100,data=WindDataTREES[Train,],control = rpart.control(cp = tree1mse$CP[CPi]))
  # prp(tree1,varlen=10)
  # ModelPlotData$tree1 <- predict(tree1,newdata = ModelPlotData)
  
  tree1mse$nsplit[CPi] <- length(tree1$splits)
  tree1mse$mse[CPi]<-mean((WindDataTREES$Power[-Train]-predict(tree1,newdata = WindDataTREES[-Train,]))^2)
  
}

## Simple Tree vs gbm
require(gbm)

tree2 <- gbm(Power~Spd100+U100+V100,data = WindDataTREES[Train,],
             n.trees = 500,
             shrinkage = 0.2,
             interaction.depth = 1,
             bag.fraction = 1,
             cv.folds = 3)
tree2perf <- gbm.perf(tree2)
# ModelPlotData$tree2 <- predict(tree2,newdata = ModelPlotData,n.trees = tree2perf)
mean((WindDataTREES$Power[-Train]-predict(tree2,newdata = WindDataTREES[-Train,],n.trees = tree2perf))^2)

tree2mse <- NULL
for(i in 1:250){
  tree2mse[i]<-mean((WindDataTREES$Power[-Train]-predict(tree2,newdata = WindDataTREES[-Train,],n.trees = i))^2)
}

plot(tree2mse,type="l",ylab="MSE",xlab="Complexity: Number of Splits (Single Tree) or Trees (GBT)",
     xlim=c(0,150))
lines(mse~nsplit,data=tree1mse,lty=2)
legend("top",c("GBT with stumps (1-split trees)","Single tree with multiple splits"),lty=1:2,bty = "n")

## GBT QR
plot(Power~Spd100,WindData,pch=".")
qs=1:9/10
for(p in qs[1:floor(length(qs)/2)]){
  gbm1 <-  gbm(Power~Spd100,data = WindDataTREES,
               n.trees = 1000,
               shrinkage = 0.05,
               interaction.depth = 1,
               bag.fraction = 0.1,
               distribution = list(name="quantile",alpha=p),
               n.minobsinnode = 1,
               cv.folds = 1)
  gbm2 <-  gbm(Power~Spd100,data = WindDataTREES,
               n.trees = 1000,
               shrinkage = 0.05,
               interaction.depth = 1,
               bag.fraction = 0.1,
               distribution = list(name="quantile",alpha=1-p),
               n.minobsinnode = 1,
               cv.folds = 1)
  
  
  polygon(c(ModelPlotData$Spd100,rev(ModelPlotData$Spd100)),
          c(predict.gbm(gbm1,newdata=ModelPlotData,n.trees=gbm.perf(gbm1,plot.it = F)),
            rev(predict.gbm(gbm2,newdata=ModelPlotData,n.trees=gbm.perf(gbm2,plot.it = F)))),
          border = F,col = adjustcolor("blue",alpha.f = p))

}


## 
i=11
hist(WindData$Power[WindData$Spd100>=i-2  ],breaks=100,
     xlab="Power",main="Histogram of Power, Wind Speed >9m/s")






